/* uxdsp.c - ALPHA interface for CHESS

  Revision: 1991-04-15

  Copyright (C) 1986, 1987, 1988, 1989, 1990 Free Software Foundation, Inc.
  Copyright (c) 1988, 1989, 1990  John Stanback

  This file is part of CHESS.

  CHESS is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY.  No author or distributor accepts responsibility to anyone for
  the consequences of using it or for whether it serves any particular
  purpose or works at all, unless he says so in writing.  Refer to the CHESS
  General Public License for full details.

  Everyone is granted permission to copy, modify and redistribute CHESS, but
  only under the conditions described in the CHESS General Public License.
  A copy of this license is supposed to have been given to you along with
  CHESS so you can know your rights and responsibilities.  It should be in a
  file named COPYING.  Among other things, the copyright notice and this
  notice must be preserved on all copies.
  
  Winboard protocol support - Jim Ablett [02-10-08]
*/

#include <ctype.h>
#include <signal.h>
#include <string.h>
#include <time.h>

#include <sys/param.h>
#include <sys/types.h>
#include <sys/file.h>


/* <stdlib.h> */
extern void *malloc (size_t);
extern void exit (int);
/* <string.h> */
extern char *strcat (char *, const char *);
extern int strcmp (const char *, const char *);
extern char *strcpy (char *, const char *);
/* <time.h> */
extern long int time (long int *);


#include "gnuchess.h"

#define pxx " PNBRQK"
#define qxx " pnbrqk"
#define rxx "12345678"
#define cxx "abcdefgh"
#define TAB (46)
#define VIR_C(s)  ((flag.reverse) ? 7-column(s) : column(s))
#define VIR_R(s)  ((flag.reverse) ? 7-row(s) : row(s))

char Analysis[128] = "";
short int MV[maxdepth];
int MSCORE;
char *DRAW;
char mvstr[4][6];
static char *ColorStr[2] =
{"White", "Black"};
static long evrate;

extern char *getenv (const char *);
void TerminateSearch (int), Die (int);

void
Initialize (void)
{
  signal (SIGINT, Die);
  signal (SIGQUIT, Die);
 
}

void
ExitChess (void)
{
  
  exit (0);
}

void
Die (int Sig)
{
  
}

void
TerminateSearch (int Sig)
{
  signal (SIGINT, SIG_IGN);
  signal (SIGQUIT, SIG_IGN);
  flag.timeout = true;
  flag.bothsides = false;
  signal (SIGINT, Die);
  signal (SIGQUIT, Die);
}
void 
ShowLine (short unsigned int *bstline)
{
}

void
algbr (short int f, short int t, short int flag)

/*
   Generate move strings in different formats.
*/

{
  int m3p;

  if (f != t)
    {
      /* algebraic notation */
      mvstr[0][0] = cxx[column (f)];
      mvstr[0][1] = rxx[row (f)];
      mvstr[0][2] = cxx[column (t)];
      mvstr[0][3] = rxx[row (t)];
      mvstr[0][4] = mvstr[3][0] = '\0';
      if (((mvstr[1][0] = pxx[board[f]]) == 'P') || (flag & promote))
	{
	  if (mvstr[0][0] == mvstr[0][2])	/* pawn did not eat */
	    {
	      mvstr[2][0] = mvstr[1][0] = mvstr[0][2];	/* to column */
	      mvstr[2][1] = mvstr[1][1] = mvstr[0][3];	/* to row */
	      m3p = 2;
	    }
	  else
	    /* pawn ate */
	    {
	      mvstr[2][0] = mvstr[1][0] = mvstr[0][0];	/* from column */
	      mvstr[2][1] = mvstr[1][1] = mvstr[0][2];	/* to column */
	      mvstr[2][2] = mvstr[0][3];
	      m3p = 3;		       /* to row */
	    }
	  mvstr[2][m3p] = mvstr[1][2] = '\0';
	  if (flag & promote)
	    {
	      mvstr[0][4] = mvstr[1][2] = mvstr[2][m3p] = qxx[flag & pmask];
	      mvstr[1][3] = mvstr[2][m3p + 1] = mvstr[0][5] = '\0';
	    }
	}
      else
	/* not a pawn */
	{
	  mvstr[2][0] = mvstr[1][0];
	  mvstr[2][1] = mvstr[0][1];
	  mvstr[2][2] = mvstr[1][1] = mvstr[0][2];	/* to column */
	  mvstr[2][3] = mvstr[1][2] = mvstr[0][3];	/* to row */
	  mvstr[2][4] = mvstr[1][3] = '\0';
	  strcpy (mvstr[3], mvstr[2]);
	  mvstr[3][1] = mvstr[0][0];
	  if (flag & cstlmask)
	    {
	      if (t > f)
		{
		  strcpy (mvstr[1], "o-o");
		  strcpy (mvstr[2], "O-O");
		}
	      else
		{
		  strcpy (mvstr[1], "o-o-o");
		  strcpy (mvstr[2], "O-O-O");
		}
	    }
	}
    }
  else
    mvstr[0][0] = mvstr[1][0] = mvstr[2][0] = mvstr[3][0] = '\0';
}

int
VerifyMove (char *s, short int iop, short unsigned int *mv)

/*
   Compare the string 's' to the list of legal moves available for the
   opponent. If a match is found, make the move on the board.
*/

{
  static short pnt, tempb, tempc, tempsf, tempst, cnt;
  static struct leaf xnode;
  struct leaf *node;

  *mv = 0;
  if (iop == 2)
    {
      UnmakeMove (opponent, &xnode, &tempb, &tempc, &tempsf, &tempst);
      return (false);
    }
  cnt = 0;
  MoveList (opponent, 2);
  pnt = TrPnt[2];
  while (pnt < TrPnt[3])
    {
      node = &Tree[pnt++];
      algbr (node->f, node->t, (short) node->flags);
      if (strcmp (s, mvstr[0]) == 0 || strcmp (s, mvstr[1]) == 0 ||
	  strcmp (s, mvstr[2]) == 0 || strcmp (s, mvstr[3]) == 0)
	{
	  cnt++;
	  xnode = *node;
	}
    }
  if (cnt == 1)
    {
      MakeMove (opponent, &xnode, &tempb, &tempc, &tempsf, &tempst, &INCscore);
      if (SqAtakd (PieceList[opponent][0], computer))
	{
	  UnmakeMove (opponent, &xnode, &tempb, &tempc, &tempsf, &tempst);
	  ShowMessage ("illegal move\n");
	  return (false);
	}
      else
	{
	  if (iop == 1)
	    return (true);
	  UpdateDisplay (xnode.f, xnode.t, 0, (short) xnode.flags);
	  if ((board[xnode.t] == pawn)
	      || (xnode.flags & capture)
	      || (xnode.flags & cstlmask))
	    {
	      Game50 = GameCnt;
	      ZeroRPT ();
	    }
	  GameList[GameCnt].depth = GameList[GameCnt].score = 0;
	  GameList[GameCnt].nodes = 0;
	  ElapsedTime (1);
	  GameList[GameCnt].time = (short) et;
	  TimeControl.clock[opponent] -= et;
	  --TimeControl.moves[opponent];
	  *mv = (xnode.f << 8) | xnode.t;
	  algbr (xnode.f, xnode.t, false);
	  return (true);
	}
    }
  if (cnt > 1)
    ShowMessage ("ambiguous move\n");
  return (false);
}

void
help (void)
{
  
}

void
EditBoard (void)

/*
  Set up a board position. Pieces are entered by typing the piece
  followed by the location. For example, Nf3 will place a knight on
  square f3.
*/

{
  short a, r, c, sq, i;
  char s[80];

 
}

void
ShowPlayers (void)
{
  if (!xboard_mode) {
  gotoXY (5, ((flag.reverse) ? 23 : 2));
  printf ("%s", (computer == black) ? "Computer" : "Human   ");
  gotoXY (5, ((flag.reverse) ? 2 : 23));
  printf ("%s", (computer == white) ? "Computer" : "Human   ");
}
}

void
ShowDepth (char ch)
{
  
  printf ("Depth= %d%c \n", Sdepth, ch);
 
}

void
ShowScore (short score)
{
  
  printf ("Score= %d\n", score);

}

void
ShowMessage (char *s)
{ 
  printf ("%s", s);

}

void
ClearMessage (void)
{
  if (!xboard_mode) {
  gotoXY (TAB, 6);
  ClrEoln ();
}
}



void
ShowCurrentMove (short int pnt, short int f, short int t)
{
  algbr (f, t, false);
 
  printf ("(%2d) %4s ", pnt, mvstr[0]);
}


void
ShowHeader (void)
{
 
  printf ("GNU Chess");
}

void
ShowSidetoMove (void)
{
  
  printf ("%2d:   %s\n", 1 + GameCnt / 2, ColorStr[player]);

}

void
ShowPrompt (void)
{
  if (!xboard_mode) {
  gotoXY (TAB, 19);
  printf ("Your move is? ");
  ClrEoln ();
}
}

void
ShowNodeCnt (long int NodeCnt, long int evrate)
{
  printf ("Nodes= %8ld, Nodes/Sec= %5ld\n", NodeCnt, evrate);
  
}

void
ShowResults (short int score, short unsigned int *bstline, char ch)
{
  unsigned char d, ply;
  if (flag.post)
    {
      ShowDepth (ch);
      ShowScore (score);
      d = 7;
      for (ply = 1; bstline[ply] > 0; ply++)
	{
 	
	  algbr ((short) bstline[ply] >> 8, (short) bstline[ply] & 0xFF, false);
	  printf ("%5s \n", mvstr[0]);
	}
      

}
}

void
SearchStartStuff (short int side)
{
  short i;

  signal (SIGINT, TerminateSearch);
  signal (SIGQUIT, TerminateSearch);
  
}

void
OutputMove (void)
{
  int i;

  UpdateDisplay (root->f, root->t, 0, (short) root->flags);
  printf("move %s\n", mvstr[0]);
        
  
  if (root->flags & draw) {
    printf ("Drawn game!\n");
    printf("game is a draw\n");
}
  else if (root->score == -9999) {
    printf ("Opponent mates!\n");
    printf("computer mates\n");
}
  else if (root->score == 9998)
    printf ("computer mates\n");
  else if (root->score < -9000)
    printf ("Opponent will soon mate!\n");
  else if (root->score > 9000)
    printf ("Computer will soon mate!\n");
  ClrEoln ();

  if (flag.post)
    {
      ShowNodeCnt (NodeCnt, evrate);
      
      for (i = 1999; i >= 0 && Tree[i].f == 0 && Tree[i].t == 0; i--) ;
      printf ("Max Tree= %5d\n", i);
     
     
     
            
    }
}

void
ElapsedTime (short int iop)

/*
  Determine the time that has passed since the search was started. If
  the elapsed time exceeds the target (ResponseTime+ExtraTime) then set
  timeout to true which will terminate the search.
*/

{
  et = time ((long *) 0) - time0;
  if (et < 0)
    et = 0;
  ETnodes += 50;
  if (et > et0 || iop == 1)
    {
      if (et > ResponseTime + ExtraTime && Sdepth > 1)
	flag.timeout = true;
      et0 = et;
      if (iop == 1)
	{
	  time0 = time ((long *) 0);
	  et0 = 0;
	}
      if (et > 0)
	/* evrate used to be Nodes / cputime I dont` know why */
	evrate = NodeCnt / (et + ft);
      else
	evrate = 0;
      ETnodes = NodeCnt + 50;
      UpdateClocks ();
    }
}

void
UpdateClocks (void)
{
  short m, s;
  m = (short) (et / 60);
  s = (short) (et - 60 * (long) m);
  if (TCflag)
    {
      m = (short) ((TimeControl.clock[player] - et) / 60);
      s = (short) (TimeControl.clock[player] - et - 60 * (long) m);
    }
  if (m < 0)
    m = 0;
  if (s < 0)
    s = 0;
  
  if (flag.post)
    ShowNodeCnt (NodeCnt, evrate);


}

void
SetTimeControl (void)
{
  if (TCflag)
    {
      TimeControl.moves[white] = TimeControl.moves[black] = TCmoves;
      TimeControl.clock[white] = TimeControl.clock[black] = 60 * (long) TCminutes;
    }
  else
    {
      TimeControl.moves[white] = TimeControl.moves[black] = 0;
      TimeControl.clock[white] = TimeControl.clock[black] = 0;
      Level = 60 * (long) TCminutes;
    }
  et = 0;
  ElapsedTime (1);
}

void
gotoXY (short int x, short int y)
{
 }

void
ClrScreen (void)
{
 // clear ();
 // refresh ();
}


void
ClrEoln (void)
{
     //   refresh ();
}


void
DrawPiece (short int sq)
{
  register char x;


}

void
ShowPostnValue (short int sq)

/*
  must have called ExaminePosition() first
*/

{
  short score;
  }

void
ShowPostnValues (void)
{
  short sq, score;

  ExaminePosition ();
  for (sq = 0; sq < 64; sq++)
    ShowPostnValue (sq);
  ScorePosition (opponent, &score);
  ShowScore (score);
}

void
UpdateDisplay (short int f, short int t, short int redraw, short int isspec)
{
  short i, sq, z;

  
}

char *InPtr;
void 
skip ()
{
  while (*InPtr != ' ')
    InPtr++;
  while (*InPtr == ' ')
    InPtr++;
}
void 
skipb ()
{
  while (*InPtr == ' ')
    InPtr++;
}
int 
parser (char *f, int side)
{
  int c1, r1, c2, r2;
  if (f[4] == 'o')
    if (side == black)
      return 0x3C3A;
    else
      return 0x0402;
  else if (f[0] == 'o')
    if (side == black)
      return 0x3C3E;
    else
      return 0x0406;
  else
    {
      c1 = f[0] - 'a';
      r1 = f[1] - '1';
      c2 = f[2] - 'a';
      r2 = f[3] - '1';
      return (locn (r1, c1) << 8) | locn (r2, c2);
    }
  return (0);
}

void
GetGame (void)
{
  FILE *fd;
  char fname[256], tname[256], *p;
  int c, i, j;
  short sq;
  unsigned short m;

  tname[0] = 0;
  if (p = getenv ("HOME"))
    strcpy (fname, p);
  else
    fname[0] = '\0';
  strcat (fname, "/");
  ShowMessage ("File name: ");
  scanf ("%s", tname);
  if (tname[0])
    strcat (fname, tname);
  else
    strcat (fname, "chess.000");
  ShowMessage ("Loading ");
  printf ("%s", fname);

  if ((fd = fopen (fname, "r")) == NULL)
    {
      ShowMessage ("Load failed");
      return;
    }

  fgets (fname, 256, fd);
  computer = opponent = white;
  InPtr = fname;
  skip ();
  if (*InPtr == 'c')
    computer = black;
  else
    opponent = black;
  skip ();
  skip ();
  skip ();
  Game50 = atoi (InPtr);
  fgets (fname, 256, fd);
  InPtr = &fname[14];
  castld[white] = ((*InPtr == 't') ? true : false);
  skip ();
  skip ();
  castld[black] = ((*InPtr == 't') ? true : false);
  fgets (fname, 256, fd);
  InPtr = &fname[11];
  skipb ();
  TCflag = atoi (InPtr);
  skip ();
  InPtr += 14;
  skipb ();
  OperatorTime = atoi (InPtr);
  fgets (fname, 256, fd);
  InPtr = &fname[11];
  skipb ();
  TimeControl.clock[white] = atoi (InPtr);
  skip ();
  skip ();
  TimeControl.moves[white] = atoi (InPtr);
  fgets (fname, 256, fd);
  InPtr = &fname[11];
  skipb ();
  TimeControl.clock[black] = atoi (InPtr);
  skip ();
  skip ();
  TimeControl.moves[black] = atoi (InPtr);
  fgets (fname, 256, fd);
  for (i = 7; i > -1; i--)
    {
      fgets (fname, 256, fd);
      p = &fname[2];
      InPtr = &fname[11];
      skipb ();
      for (j = 0; j < 8; j++)
	{
	  sq = i * 8 + j;
	  if (*p == '.')
	    {
	      board[sq] = no_piece;
	      color[sq] = neutral;
	    }
	  else
	    {
	      for (c = 0; c < 8; c++)
		{
		  if (*p == pxx[c])
		    {
		      board[sq] = c;
		      color[sq] = black;
		    }
		}
	      for (c = 0; c < 8; c++)
		{
		  if (*p == qxx[c])
		    {
		      board[sq] = c;
		      color[sq] = white;
		    }
		}
	    }
	  p++;
	  Mvboard[sq] = atoi (InPtr);
	  skip ();
	}
    }
  GameCnt = 0;
  fgets (fname, 256, fd);
  fgets (fname, 256, fd);
  fgets (fname, 256, fd);
  while (fgets (fname, 256, fd))
    {
      struct GameRec *g;
      int side = computer;
      side = otherside[side];
      ++GameCnt;
      InPtr = fname;
      skipb ();
      g = &GameList[GameCnt];
      g->gmove = parser (InPtr, side);
      skip ();
      g->score = atoi (InPtr);
      skip ();
      g->depth = atoi (InPtr);
      skip ();
      g->nodes = atoi (InPtr);
      skip ();
      g->time = atoi (InPtr);
      skip ();
      g->flags = c = atoi (InPtr);
      g->piece = no_piece;
      g->color = neutral;
      if (c & (capture | cstlmask))
	{
	  if (c & capture)
	    {
	      skip ();
	      for (c = 0; c < 8; c++)
		if (pxx[c] == *InPtr)
		  break;
	      g->piece = c;
	    }
	  skip ();
	  g->color = ((*InPtr == 'B') ? black : white);
	}
    }
  /* GameCnt--;*/
  if (TimeControl.clock[white] > 0)
    TCflag = true;
  fclose (fd);
  ZeroRPT ();
  InitializeStats ();
  UpdateDisplay (0, 0, 1, 0);
  Sdepth = 0;
}


void
GetXGame (void)
{
  FILE *fd;
  char fname[256], *p, tname[256];
  int c, i, j;
  short sq;
  unsigned short m;
  tname[0] = 0;
  ShowMessage ("File name: ");
  scanf ("%s", tname);
  if (tname[0])
    strcat (fname, tname);
  else
    strcpy (fname, "xboard.position.read");
  ShowMessage ("Loading ");
  printf ("%s", fname);

  if ((fd = fopen (fname, "r")) != NULL)
    {
      NewGame ();
      fgets (fname, 256, fd);
      fname[6] = '\0';
      if (strcmp (fname, "xboard"))
	return;
      fgets (fname, 256, fd);
      fgets (fname, 256, fd);
      for (i = 7; i > -1; i--)
	{
	  fgets (fname, 256, fd);
	  p = fname;
	  for (j = 0; j < 8; j++)
	    {
	      sq = i * 8 + j;
	      if (*p == '.')
		{
		  board[sq] = no_piece;
		  color[sq] = neutral;
		}
	      else
		{
		  for (c = 0; c < 8; c++)
		    {
		      if (*p == qxx[c])
			{
			  board[sq] = c;
			  color[sq] = black;
			}
		    }
		  for (c = 0; c < 8; c++)
		    {
		      if (*p == pxx[c])
			{
			  board[sq] = c;
			  color[sq] = white;
			}
		    }
		}
	      p += 2;
	    }
	}
      fclose (fd);
    }
  ZeroRPT ();
  InitializeStats ();
  UpdateDisplay (0, 0, 1, 0);
  Sdepth = 0;
}

void
SaveGame (void)
{
  FILE *fd;
  char fname[256], tname[256], *tmp;
  short sq, i, c, f, t;
  char p;
  char *b, *w;

  tname[0] = 0;

  if (tmp = getenv ("HOME"))
    strcpy (fname, tmp);
  else
    fname[0] = '\0';
  strcat (fname, "/");

  ShowMessage ("File name: ");
  //refresh ();
  scanf ("%s", tname);

  if (tname[0])
    strcat (fname, tname);
  else
    strcat (fname, "chess.000");

  ShowMessage ("Saving ");
  printf ("%s", fname);

  if (NULL == (fd = fopen (fname, "w")))
    {
      ShowMessage ("Not saved");
      return;
    }
  b = w = "Human";
  if (computer == black)
    b = "computer";
  if (computer == white)
    w = "computer";
  fprintf (fd, "Black %s White %s %d\n", b, w, Game50);
  fprintf (fd, "Castled White %s Black %s\n", castld[white] ? "true" : "false", castld[black] ? "true" : "false");
  fprintf (fd, "TimeControl %d Operator Time %d\n", TCflag, OperatorTime);
  fprintf (fd, "White Clock %ld Moves %ld\nBlack Clock %d Moves %d\n\n",
	   TimeControl.clock[white], TimeControl.moves[white],
	   TimeControl.clock[black], TimeControl.moves[black]);
  for (i = 7; i > -1; i--)
    {
      fprintf (fd, "%1d ", i + 1);
      for (c = 0; c < 8; c++)
	{
	  sq = i * 8 + c;
	  switch (color[sq])
	    {
	    case black:
	      p = pxx[board[sq]];
	      break;
	    case white:
	      p = qxx[board[sq]];
	      break;
	    default:
	      p = '.';
	    }
	  fprintf (fd, "%c", p);
	}
      for (f = i * 8; f < i * 8 + 8; f++)
	fprintf (fd, " %d", Mvboard[f]);
      fprintf (fd, "\n");
    }
  fprintf (fd, "  %s\n", cxx);
  fprintf (fd, "\nmove  score depth  nodes  time flags capture color\n");
  for (i = 1; i <= GameCnt; i++)
    {
      struct GameRec *g = &GameList[i];
      f = g->gmove >> 8;
      t = (g->gmove & 0xFF);
      algbr (f, t, g->flags);
      fprintf (fd, "%s %5d %5d %7ld %5d %5d   %c   %s\n",
	       mvstr[0], g->score, g->depth,
	       g->nodes, g->time, g->flags,
	       pxx[g->piece], g->color == 2 ? "     " : ColorStr[g->color]);
    }
  fclose (fd);
}


void
ListGame (void)
{
  FILE *fd;
  char fname[256];
  char *tmp;
  short i, f, t;

  if (tmp = getenv ("HOME"))
    strcpy (fname, tmp);
  else
    fname[0] = '\0';
  strcat (fname, "/chess.lst");

  if (fd = fopen (fname, "w"))
    ShowMessage ("Writing ~/chess.lst");
  else
    {
      ShowMessage ("Cannot write ~/chess.lst");
      return;
    }

  fprintf (fd, "gnuchess\n");
  fprintf (fd, "       score  depth   nodes  time         ");
  fprintf (fd, "       score  depth   nodes  time\n");
  for (i = 1; i <= GameCnt; i++)
    {
      f = GameList[i].gmove >> 8;
      t = (GameList[i].gmove & 0xFF);
      algbr (f, t, false);
      if ((i % 2) == 0)
	fprintf (fd, "\n");
      else
	fprintf (fd, "         ");
      fprintf (fd, "%5s  %5d     %2d %7ld %5d", mvstr[0],
	       GameList[i].score, GameList[i].depth,
	       GameList[i].nodes, GameList[i].time);
    }
  fprintf (fd, "\n\n");
  fclose (fd);
  ShowMessage ("~/chess.lst written");
}

void
Undo (void)

/*
  Undo the most recent half-move.
*/

{
  short f, t;
  f = GameList[GameCnt].gmove >> 8;
  t = GameList[GameCnt].gmove & 0xFF;
  if (board[t] == king && distance (t, f) > 1)
    (void) castle (GameList[GameCnt].color, f, t, 2);
  else
    {
      /* Check for promotion: */
      if (GameList[GameCnt].flags & promote)
	{
	  board[t] = pawn;
	}
      board[f] = board[t];
      color[f] = color[t];
      board[t] = GameList[GameCnt].piece;
      color[t] = GameList[GameCnt].color;
      if (color[t] != neutral)
	Mvboard[t]--;
      Mvboard[f]--;
    }
  if (GameList[GameCnt].flags & epmask)
    EnPassant (color[t], f, t, 2);
  if (TCflag)
    ++TimeControl.moves[color[f]];
  GameCnt--;
  computer = otherside[computer];
  opponent = otherside[opponent];
  flag.mate = false;
  Sdepth = 0;
  player = otherside[player];
  if (!xboard_mode) {
  ShowSidetoMove();
  UpdateDisplay (0, 0, 1, 0);
}
  InitializeStats ();
}

void
ChangeAlphaWindow (void)
{
  if (!xboard_mode) {
  ShowMessage ("Awindow= ");
  scanf ("%hd", &Awindow);
}
}

void
ChangeBetaWindow (void)
{
  if (!xboard_mode) {
  ShowMessage ("Bwindow= ");
  scanf ("%hd", &Bwindow);
}
}

void
GiveHint (void)
{
  if (!xboard_mode) {
  char s[40];
  algbr ((short) (hint >> 8), (short) (hint & 0xFF), false);
  strcpy (s, "try ");
  strcat (s, mvstr[0]);
  ShowMessage (s);
}
}

void
ChangeHashDepth (void)
{
  if (!xboard_mode) {
  ShowMessage ("hashdepth= ");
  scanf ("%hd", &HashDepth);
  ShowMessage ("MoveLimit= ");
  scanf ("%hd", &HashMoveLimit);
}
}

void
ChangeSearchDepth (void)
{
  if (!xboard_mode) {
  ShowMessage ("depth= ");
  scanf ("%hd", &MaxSearchDepth);
}
}

void
SetContempt (void)
{
  if (!xboard_mode) {
  ShowMessage ("contempt= ");
  scanf ("%hd", &contempt);
}
}

void
ChangeXwindow (void)
{
  if (!xboard_mode) {
  ShowMessage ("xwndw= ");
  scanf ("%hd", &xwndw);
}
}

void
SelectLevel (void)
{
  if (!xboard_mode) {
  ClrScreen ();
  gotoXY (32, 2);
  printf ("CHESS");
  gotoXY (20, 4);
  printf (" 1.   60 moves in   5 minutes");
  gotoXY (20, 5);
  printf (" 2.   60 moves in  15 minutes");
  gotoXY (20, 6);
  printf (" 3.   60 moves in  30 minutes");
  gotoXY (20, 7);
  printf (" 4.   40 moves in  30 minutes");
  gotoXY (20, 8);
  printf (" 5.   40 moves in  60 minutes");
  gotoXY (20, 9);
  printf (" 6.   40 moves in 120 minutes");
  gotoXY (20, 10);
  printf (" 7.   40 moves in 240 minutes");
  gotoXY (20, 11);
  printf (" 8.    1 move  in  15 minutes");
  gotoXY (20, 12);
  printf (" 9.    1 move  in  60 minutes");
  gotoXY (20, 13);
  printf ("10.    1 move  in 600 minutes");

  OperatorTime = 1;
  TCmoves = 60;
  TCminutes = 5;

  gotoXY (20, 17);
  printf ("Enter Level: ");
//  refresh ();
  scanf ("%ld", &Level);
  switch ((int) Level)
    {
    case 1:
      TCmoves = 60;
      TCminutes = 5;
      break;
    case 2:
      TCmoves = 60;
      TCminutes = 15;
      break;
    case 3:
      TCmoves = 60;
      TCminutes = 30;
      break;
    case 4:
      TCmoves = 40;
      TCminutes = 30;
      break;
    case 5:
      TCmoves = 40;
      TCminutes = 60;
      break;
    case 6:
      TCmoves = 40;
      TCminutes = 120;
      break;
    case 7:
      TCmoves = 40;
      TCminutes = 240;
      break;
    case 8:
      TCmoves = 1;
      TCminutes = 15;
      break;
    case 9:
      TCmoves = 1;
      TCminutes = 60;
      break;
    case 10:
      TCmoves = 1;
      TCminutes = 600;
      break;
    }

  TCflag = (TCmoves > 1);
  SetTimeControl ();
  ClrScreen ();
  UpdateDisplay (0, 0, 1, 0);
}

  
}

void
DoDebug (void)
{
  short c, p, sq, tp, tc, tsq, score;
  char s[40];

  ExaminePosition ();
  ShowMessage ("Enter piece: ");
  scanf ("%s", s);
  c = neutral;
  if (s[0] == 'w' || s[0] == 'W')
    c = white;
  if (s[0] == 'b' || s[0] == 'B')
    c = black;
  for (p = king; p > no_piece; p--)
    if ((s[1] == pxx[p]) || (s[1] == qxx[p]))
      break;
  for (sq = 0; sq < 64; sq++)
    {
      tp = board[sq];
      tc = color[sq];
      board[sq] = p;
      color[sq] = c;
      tsq = PieceList[c][1];
      PieceList[c][1] = sq;
      ShowPostnValue (sq);
      PieceList[c][1] = tsq;
      board[sq] = tp;
      color[sq] = tc;
    }
  ScorePosition (opponent, &score);
  ShowScore (score);
}

void
TestSpeed (void (*f) (short int side, short int ply))
{
  short i;
  long t1, t2;

  t1 = time (0);
  for (i = 0; i < 10000; i++)
    {
      f (opponent, 2);
    }
  t2 = time (0);
  NodeCnt = 10000L * (TrPnt[3] - TrPnt[2]);
  evrate = NodeCnt / (t2 - t1);
  ShowNodeCnt (NodeCnt, evrate);
}

void
InputCommand (void)

/*
  Process the users command. If easy mode is OFF (the computer is thinking
  on opponents time) and the program is out of book, then make the 'hint'
  move on the board and call SelectMove() to find a response. The user
  terminates the search by entering ^C (quit siqnal) before entering a
  command. If the opponent does not make the hint move, then set Sdepth to
  zero.
*/

{
  short ok, tmp;
  unsigned short mv;
  char s[256];
  char line[256];
  char command[256];
  
  ok = flag.quit = false;
  player = opponent;
  
  
  ft = 0;
  
  /*
  if (hint > 0 && !flag.easy && Book == NULL)
    {
      fflush (stdout);
      time0 = time ((long *) 0);
      algbr ((short) hint >> 8, (short) hint & 0xFF, false);
      strcpy (s, mvstr[0]);
      tmp = epsquare;
      
      if (VerifyMove (s, 1, &mv)) {
	  
	  SelectMove (computer, 2);
	  (void) VerifyMove (mvstr[0], 2, &mv);
	  if (Sdepth > 0)
	    Sdepth--;
	}
      ft = time ((long *) 0) - time0;
      epsquare = tmp;
    }
    */
    
  signal (SIGINT, Die);
  signal (SIGQUIT, Die);


  while (!(ok || flag.quit))
    {
      
    if (!fgets(line, 256, stdin)) return;


		if (line[0] == '\n') continue;
		
                      
            sscanf(line, "%s", command); 
    
    
      player = opponent;
     
     
      ok = VerifyMove (command, 0, &mv);
     
     
     
      if (ok && mv != hint)
	{
	  Sdepth = 0;
	  ft = 0;
	}
	
      
	
      else if ((strcmp (command, "quit") == 0) || (strcmp (command, "exit") == 0))
	exit(0);
      else if (strcmp (command, "post") == 0)
	flag.post = !flag.post;
      else if (strcmp (command, "edit") == 0)
	EditBoard ();
      else if (strcmp (command, "go") == 0)
	ok = true;
      else if (strcmp (command, "help") == 0)
	help ();
      else if (strcmp (command, "force") == 0)
	flag.force = !flag.force;
	
	  else if (strcmp(command, "protover") == 0) {
              printf("feature colors=1\n");
              printf("feature myname=\"GnuChess %s by J.Stanback/S.Cracraft\"\n",VERSION);
              printf("feature time=1\n");
              printf("feature pause=0\n");
              printf("feature sigint=0\n");
              printf("feature sigterm=0\n");
              printf("feature done=1\n");
              }
      
      
      else if (!strcmp (command, "level")) {
           
      sscanf(line, "level %d %d &d", &maxmoves,&maxtime, &inctime);     	
      
      if (!maxmoves) maxmoves = 40;
      if (!maxtime) maxtime = 3;
       
      TCmoves = maxmoves;
      TCminutes = maxtime;
      
      OperatorTime = 1;
	  TCflag = (TCmoves > 1);
      SetTimeControl ();
      printf("[gnuchess] time control = %d moves in %d mins %d increment\n", TCmoves, TCminutes, inctime);
  }	 
  
    else if (!strcmp(command, "result")) {
           char result[256];
           sscanf(line,"result %s",result);
           flag.force = true;
           }

  
      else if (strcmp (command, "book") == 0)
	Book = NULL;
      else if (strcmp (command, "undo") == 0 && GameCnt > 0)
	Undo ();
      else if (strcmp (command, "new") == 0)
	NewGame ();
      else if (strcmp (command, "list") == 0)
	ListGame ();
      else if (strcmp (command, "hash") == 0)
	flag.hash = !flag.hash; 
      else if (strcmp (command, "Awindow") == 0)
	ChangeAlphaWindow ();
      else if (strcmp (command, "Bwindow") == 0)
	ChangeBetaWindow ();
      else if (strcmp (command, "hint") == 0)
	GiveHint ();
      else if (strcmp (command, "both") == 0)
	{
	  flag.bothsides = !flag.bothsides;
	  Sdepth = 0;
	  SelectMove (opponent, 1);
	  ok = true;
	}
    
      else if (strcmp (command, "switch") == 0)
	{
	  computer = otherside[computer];
	  opponent = otherside[opponent];
	  flag.force = false;
	  Sdepth = 0;
	  ok = true;
	}
      else if (strcmp (command, "white") == 0)
	{
	  computer = white;
	  opponent = black;
	  flag.force = false;
	  Sdepth = 0;
	  /*ok = true;/* dont start automatically on white command */
	}
      else if (strcmp (command, "black") == 0)
	{
	  computer = black;
	  opponent = white;
	  flag.force = false;
	  Sdepth = 0;
	  /*ok = true;/* dont start automatically on white command */
	}
      else if (strcmp (command, "remove") == 0 && GameCnt > 1)
	{
	  Undo ();
	  Undo ();
	}
      else if (strcmp (command, "get") == 0)
	GetGame ();
      else if (strcmp (command, "xget") == 0)
	GetXGame ();
      else if (strcmp (command, "save") == 0)
	SaveGame ();
      else if (strcmp (command, "depth") == 0)
	ChangeSearchDepth ();
      else if (strcmp (command, "hashdepth") == 0)
	ChangeHashDepth ();
      else if (strcmp (command, "random") == 0)
	dither = 6;
      else if (strcmp (command, "easy") == 0)
	flag.easy = !flag.easy;
      else if (strcmp (command, "contempt") == 0)
	SetContempt ();
      else if (strcmp (command, "xwndw") == 0)
	ChangeXwindow ();
	else if (strcmp (command, "xboard") == 0) {
	xboard_mode=true;
	printf("feature done=0\n");
}
	
      else if (strcmp (command, "test") == 0)
	{
	  ShowMessage ("Testing MoveList Speed");
	  TestSpeed (MoveList);
	  ShowMessage ("Testing CaptureList Speed");
	  TestSpeed (CaptureList);
	}
      else if (strcmp (command, "p") == 0)
	ShowPostnValues ();
      else if (strcmp (command, "debug") == 0)
	DoDebug ();
    }

 if (!xboard_mode)  ClearMessage ();
  ElapsedTime (1);
  if (flag.force)
    {
      computer = opponent;
      opponent = otherside[computer];
    }
  signal (SIGINT, TerminateSearch);
  signal (SIGQUIT, TerminateSearch);

}

